#!/python/bin

''' Script to modify the TMC file. This scripts change the file names
    of the ACS modes that support MJD. In these cases the compnent that
    does not have MJD should use the MJD synphot file.  MKCOMPTAB cannot
    be changed to do this kind of maniputlation (it would be too much work),
    and pysynphot cannot handle to use the same component for MJD and not MJD
    calculations. Therefore, the only way in which ACS can submit one file
    to cover both cases is if the TMC file is changed manually.

    For example:

        COMPNAME = acs_wfc_ccd1 uses File = cracscomp$acs_wfc_ccd1_019_syn.fits
        COMPNAME = acs_wfc_ccd1_mjd uses File = cracscomp$acs_wfc_ccd1_mjd_019_syn.fits[mjd#]

    This means that even file acs_wfc_ccd1_019_syn.fits and
    acs_wfc_ccd1_mjd_019_syn.fits have the same data, they need to have
    different header keyword COMPNAME so the MKCOMPTAB script can add the [mjd#]
    property at the end of the file name (wich in turn pysynphot knows to use
            for thye parameterized calculations).  On the other hand, pysynphot
    will use the file that is given by the column FILENAME for the given
    COMPNAME in the TMC table. It does not care if both are the same, as long
    as one of the declarations has [mjd#] at the end, and the file itself has
    several columns to support parameteriztaion. Since both files
    acs_wfc_ccd1_019_syn.fits and acs_wfc_ccd1_mjd_019_syn.fits have the same
    columsn and data, any of them can use for MJD or not MJD calculations.

    Since MKCOMPTAB cannot handle this change, I wrote this script to look for
    the COMPNAMES affected and change the name of the FILENAME for the non
    MJD COMPNAME to that of the MJD one; in this case pysynphot will use the
    default column.

    By doing this, the ACS team can deliver only the MJD file.

    November 02, 2012 update by R. Diaz
    -------------------------------------------------------------------------

    MKCOMPTAB cannot add the "[mjd#]" part to the end of the new MJD files for
    COS and WFC3. This program looks for the components that are MJD and adds
    this extension to the filename.

    HISTORY: In order to document these changes, I added a parameter --hist
    to input an ASCII file with the history that will be added to the file.
    If there is no history file given, it is going to take "histroy.txt" as the
    default.


'''

import pyfits as py
import calendar
from optparse import OptionParser
import os
import time

# List of modes that should be changed
#acs
acs_changes = ['acs_hrc_ccd','acs_wfc_ccd1','acs_wfc_ccd2']
acs_mjd= ['acs_hrc_ccd_mjd','acs_wfc_ccd1_mjd','acs_wfc_ccd2_mjd']
#cos
cos_mjd=['cos_mirrora_m','cos_mirrorb_m','cosmcp_g130mc1055m','cosmcp_g130mc1096m',\
        'cosmcp_g130mc1222m','cosmcp_g130mc1291m','cosmcp_g130mc1300m','cosmcp_g130mc1309m',\
        'cosmcp_g130mc1318m','cosmcp_g130mc1327m','cosmcp_g140lc1105m','cosmcp_g140lc1230m',\
        'cosmcp_g140lc1280m','cosmcp_g160mc1577m','cosmcp_g160mc1589m','cosmcp_g160mc1600m',\
        'cosmcp_g160mc1611m','cosmcp_g160mc1623m','cosncm3g185mc1786m','cosncm3g185mc1817m',\
        'cosncm3g185mc1835m','cosncm3g185mc1850m','cosncm3g185mc1864m','cosncm3g185mc1882m',\
        'cosncm3g185mc1890m','cosncm3g185mc1900m','cosncm3g185mc1913m','cosncm3g185mc1921m',\
        'cosncm3g185mc1941m','cosncm3g185mc1953m','cosncm3g185mc1971m','cosncm3g185mc1986m',\
        'cosncm3g185mc2010m','cosncm3g225mc2186m','cosncm3g225mc2217m','cosncm3g225mc2233m',\
        'cosncm3g225mc2250m','cosncm3g225mc2268m','cosncm3g225mc2283m','cosncm3g225mc2306m',\
        'cosncm3g225mc2325m','cosncm3g225mc2339m','cosncm3g225mc2357m','cosncm3g225mc2373m',\
        'cosncm3g225mc2390m','cosncm3g225mc2410m','cosncm3g230lc2635m','cosncm3g230lc2950m',\
        'cosncm3g230lc3000m','cosncm3g230lc3360m','cosncm3g285mc2617m','cosncm3g285mc2637m',\
        'cosncm3g285mc2657m','cosncm3g285mc2676m','cosncm3g285mc2695m','cosncm3g285mc2709m',\
        'cosncm3g285mc2719m','cosncm3g285mc2739m','cosncm3g285mc2850m','cosncm3g285mc2952m',\
        'cosncm3g285mc2979m','cosncm3g285mc2996m','cosncm3g285mc3018m','cosncm3g285mc3035m',\
        'cosncm3g285mc3057m','cosncm3g285mc3074m','cosncm3g285mc3094m']
#wfc3
wfc3_mjd=['wfc3_ir_f098m_m','wfc3_ir_f105w_m','wfc3_ir_f110w_m','wfc3_ir_f125w_m',\
        'wfc3_ir_f126n_m','wfc3_ir_f127m_m','wfc3_ir_f128n_m','wfc3_ir_f130n_m','wfc3_ir_f132n_m',\
        'wfc3_ir_f139m_m','wfc3_ir_f140w_m','wfc3_ir_f153m_m','wfc3_ir_f160w_m','wfc3_ir_f164n_m',\
        'wfc3_ir_f167n_m','wfc3_ir_g102_src_m','wfc3_ir_g141_src_m','wfc3_uvis_f200lp_m',\
        'wfc3_uvis_f218w_m','wfc3_uvis_f225w_m','wfc3_uvis_f275w_m','wfc3_uvis_f280n_m',\
        'wfc3_uvis_f300x_m','wfc3_uvis_f336w_m','wfc3_uvis_f343n_m','wfc3_uvis_f350lp_m',\
        'wfc3_uvis_f373n_m','wfc3_uvis_f390m_m','wfc3_uvis_f390w_m','wfc3_uvis_f395n_m',\
        'wfc3_uvis_f410m_m','wfc3_uvis_f438w_m','wfc3_uvis_f467m_m','wfc3_uvis_f469n_m',\
        'wfc3_uvis_f475w_m','wfc3_uvis_f475x_m','wfc3_uvis_f487n_m','wfc3_uvis_f502n_m',\
        'wfc3_uvis_f547m_m','wfc3_uvis_f555w_m','wfc3_uvis_f600lp_m','wfc3_uvis_f606w_m',\
        'wfc3_uvis_f621m_m','wfc3_uvis_f625w_m','wfc3_uvis_f631n_m','wfc3_uvis_f645n_m',\
        'wfc3_uvis_f656n_m','wfc3_uvis_f657n_m','wfc3_uvis_f658n_m','wfc3_uvis_f665n_m',\
        'wfc3_uvis_f673n_m','wfc3_uvis_f680n_m','wfc3_uvis_f689m_m','wfc3_uvis_f763m_m',\
        'wfc3_uvis_f775w_m','wfc3_uvis_f814w_m','wfc3_uvis_f845m_m','wfc3_uvis_f850lp_m',\
        'wfc3_uvis_f953n_m','wfc3_uvis_fq232n_m','wfc3_uvis_fq243n_m','wfc3_uvis_fq378n_m',\
        'wfc3_uvis_fq387n_m','wfc3_uvis_fq422m_m','wfc3_uvis_fq436n_m','wfc3_uvis_fq437n_m',\
        'wfc3_uvis_fq492n_m','wfc3_uvis_fq508n_m','wfc3_uvis_fq575n_m','wfc3_uvis_fq619n_m',\
        'wfc3_uvis_fq634n_m','wfc3_uvis_fq672n_m','wfc3_uvis_fq674n_m','wfc3_uvis_fq727n_m',\
        'wfc3_uvis_fq750n_m','wfc3_uvis_fq889n_m','wfc3_uvis_fq906n_m','wfc3_uvis_fq924n_m',\
        'wfc3_uvis_fq937n_m','wfc3_uvisg280src_m']

def parse_args():
    '''
    Parse command line arguments
    '''
    parser = OptionParser()
    parser.add_option('--input', dest='infile',  help = 'Input file, default: new_tmc.fits',default='new_tmc.fits')
    parser.add_option('--output', dest='outfile',help = 'Output file, default: new_mod_tmc.fits', default='new_mod_tmc.fits')
    (options, args) = parser.parse_args()
    return options, args

#Determines today's date.
def UseafterToday():
    ''' Takes the current day ad uses it to fill the useafter_var
    '''
    months=dict((k,v) for k,v in enumerate(calendar.month_abbr))
    localtime = time.localtime(time.time())
    n1=int(localtime.tm_mon)
    return months[n1] + ' ' + str(localtime.tm_mday) + ' ' + str(localtime.tm_year)

#Makes changes to the ACS modes.
def change_acs(data):
    f_name = []
    file_name = data.field('filename')
    # Finds the ACS files that have mjd in their name, given in the array
    # acs_mjd
    for i in xrange(len(acs_mjd)):
        filename = [data[j].field('filename') for j in
            xrange(len(data)) if data[j].field('compname') ==
            acs_mjd[i]]
        # finds the position of the mjd argument in the file name
        # Saves it up to the point before the "[mjd#]" extension and
        # saves it
        l1=filename[0].find('mjd')
        filename = filename[0:l1+16]
        f_name.append(filename)

    #print f_name
    # Looks for the components that will be replaced  and changes the value of
    # filename with the name of the mjd files but withouth the "[mjd#]"
    for i in xrange(len(acs_changes)):
        for idx in xrange(len(file_name)):
            if data[idx].field('compname') == acs_changes[i]:
                #data[idx].field('compname')
                l1=len(f_name[i][0])
                data['filename'][idx] = f_name[i][0][0:l1-6]

    return data

# Adds [mjd#] to the filenames
def add_mjd(data, comp, comp_mjd):
    fname_new=data
    #for i in xrange(len(data)):
    #    fname_new.append(data[i].field('filename'))
    
    #fname_new = data.field('filename')
    for i in xrange(len(comp_mjd)):
        for j in xrange(len(fname_new)):
            if comp[j] == comp_mjd[i]:
                fname_new[j] = fname_new[j] + '[mjd#]'
            	#print fname_new[j]

    #d1 = [m for m in data.field('filename') if re.search('wfc3',m)]

    #return data
    return fname_new

#d1 = [m for m in data.field('filename') if re.search('wfc3',m)]
#

if __name__ == "__main__":
    options, args = parse_args()

    if options.outfile:
        # opens the file for reading only
        file1 = py.open(options.infile)
    else: # if no output file given opens the input file for modification
        file1 = py.open(options.infile, mode='update')

    #reads the file and just changes the column format and  descrip
    table = file1[1]
    new_columns = py.ColDefs(table.columns)
    new_columns.info()
    

    # The changes
    new_columns[2].format = '56A'
    new_columns[2].disp = 'A-56'
    new_table = py.new_table(new_columns, nrows = (len(table.data)))
    file1[1] = new_table
    file1.close()

    #re-opens the file this time to change the data
    if options.outfile:
        # opens the file for reading only
        file1 = py.open(options.infile)
    else:
        # if no output file given opens the input file for modification
        file1 = py.open(options.infile, mode='update')

    #gets the data
    hdr = file1[0].header
    data = table.data
    new_columns = py.ColDefs(table.columns)

    #extract useafter date from file
    mm=UseafterToday()
    file1[0].header.update('useafter', UseafterToday())

    #gets the new data array with the filenames changed for ACS
    data=change_acs(data)

    #gets the new data array with the filenames changed for COS
    file_name0=[]
    for i in xrange(len(data.field('filename'))):
       file_name0.append(data[i].field('filename'))
    file_name1=add_mjd(file_name0, data.field('compname'), cos_mjd)


    #gets the new data array with the filenames changed for WfC3
    file_name=add_mjd(file_name1,  data.field('compname'), wfc3_mjd)

    col1 = py.Column(name=new_columns[0].name, format=new_columns[0].format, disp= 'A-26', array=data.field('TIME'))        
    col2 = py.Column(name=new_columns[1].name, format=new_columns[1].format, array=data.field('COMPNAME'))
    col3 = py.Column(name=new_columns[2].name, format='56A', disp = 'A-56', array=file_name)
    col4 = py.Column(name=new_columns[3].name, format=new_columns[3].format, array=data.field('COMMENT'))

    cols= [col1, col2, col3, col4]

    new_table = py.new_table(cols, nrows = (len(table.data)))

    # Reads the history input from a file
    #
    # Adds history to document this change
    file1[0].header.add_history(' ')
    file1[0].header.add_history(str(UseafterToday()))
    file1[0].header.add_history(' ')
    file1[0].header.add_history('File updated to add the [#MJD] parameter to MJD files as well as')
    file1[0].header.add_history('to correct the ACS files.')

    #if options.outfile:
    if os.access(options.outfile, os.R_OK):
        os.remove(options.outfile)
    if options.outfile:
        hdu = py.HDUList()
        hdu.append(file1[0])
        hdu.append(new_table)
        hdu.verify()
        print options.outfile
        hdu.writeto(options.outfile)
    else:
        file1[1] = new_table

    #Otherwise colosing the input file will automaticaly update the table
    # If an output files was given, the file will be closed normaly
    file1.close()
